var levelconfig = {};
const fs = require("fs");
const http = require("http");
const qrimg = require("qr-image");
const express = require("express");
const rimraf = require("rimraf");
const moment = require("moment-timezone");
const bodyParser = require("body-parser");
const publicIp = require("ip");
const cors = require("cors");
const spintax = require("spintax");
const Boom = require("@hapi/boom");
const P = require("pino");
const app = express();
const axios = require("axios");
const server = http.createServer(app);
const { Server } = require("socket.io");
const config = require("./../config.js");
const Common = require("./common.js");
const Extend = require("./extend.js");
const bulks = {};
const total_contacts = {};
const chatbots = {};
const limit_messages = {};
const stats_history = {};
const sessions = {};
const new_sessions = {};
const session_dir = __dirname + "/../sessions/";
let verify_next = 0;
let verify_response = false;
let verified = false;
let chatbot_delay = 1000;
let wa_version = "";

const io = new Server(server, {
  cors: {
    origin: config.frontend,
  },
});

app.use(
  bodyParser.urlencoded({
    extended: true,
    limit: "50mb",
  })
);
io.on("connection", () => {
  console.log("new connection");
});
const {
  default: makeWASocket,
  BufferJSON,
  useMultiFileAuthState,
  DisconnectReason,
  fetchLatestWaWebVersion,
  WAMessageStubType,
} = require("@whiskeysockets/baileys");
const WAZIPER = {
  io: io,
  app: app,
  server: server,
  cors: cors(config.cors),
  makeWASocket: async function (instanceId) {
    const { state: state, saveCreds: saveCreds } = await useMultiFileAuthState(
      "sessions/" + instanceId
    );
    levelconfig.level = "silent";
    const session = makeWASocket({
      auth: state,
      printQRInTerminal: false,
      version: wa_version,
      browser: [instanceId, "Safari", "16.5"],
      logger: P(levelconfig),
      receivedPendingNotifications: false,
      defaultQueryTimeoutMs: undefined,
    });
    session.ev.process(async (data) => {
      if (data["connection.update"]) {
        const connectionUpdate = data["connection.update"];
        const {
          connection: connection,
          lastDisconnect: lastDisconnect,
          isNewLogin: isNewLogin,
          qr: qr,
          receivedPendingNotifications: receivedPendingNotifications,
        } = connectionUpdate;
        if (qr != undefined) {
          session.qrcode = qr;
          if (new_sessions[instanceId] == undefined) {
            new_sessions[instanceId] = new Date().getTime() / 1000 + 300;
          }
        }
        if (isNewLogin) {
          await WAZIPER.makeWASocket(instanceId);
          return;
        }
        if (lastDisconnect != undefined && lastDisconnect.error != undefined) {
          var statusCode = lastDisconnect.error.output.statusCode;
          if (
            DisconnectReason.restartRequired == statusCode ||
            DisconnectReason.connectionClosed == statusCode
          ) {
            console.log("recreando WASocket reason:", statusCode);
            await WAZIPER.makeWASocket(instanceId);
          }
        }
        switch (connection) {
          case "close":
            var isReconnecting =
              lastDisconnect.error?.output?.statusCode !==
                DisconnectReason.loggedOut ||
              lastDisconnect.error?.output?.statusCode == 0;
            if (
              lastDisconnect.error?.output?.statusCode ==
                DisconnectReason.timedOut &&
              (sessions[instanceId]?.qrcode || new_sessions[instanceId])
            ) {
              isReconnecting = false;
            }
            console.log(
              "connection closed due to ",
              lastDisconnect.error?.output?.statusCode,
              ", reconnecting ",
              isReconnecting
            );
            if (!isReconnecting) {
              var sessionDir = session_dir + instanceId;
              if (fs.existsSync(sessionDir)) {
                rimraf.sync(sessionDir);
                delete sessions[instanceId];
                delete chatbots[instanceId];
                delete bulks[instanceId];
              }
              delete new_sessions[instanceId];
              await WAZIPER.logout(instanceId);
            } else {
              console.log("creating new session for", instanceId);
              delete sessions[instanceId];
              await new Promise((resolve) => setTimeout(resolve, 5000));
              await WAZIPER.session(instanceId);
              console.log("new session was created for", instanceId);
            }
            break;
          case "open":
            if (session.user.name == undefined) {
              await Common.sleep(3000);
              await WAZIPER.makeWASocket(instanceId);
              break;
            }
            sessions[instanceId] = session;
            if (sessions[instanceId].qrcode != undefined) {
              console.log("remove new session qr");
              delete sessions[instanceId].qrcode;
              delete new_sessions[instanceId];
            }
            var instanceObj = {
              instance_id: instanceId,
            };
            var statusObj = {
              status: 0,
            };
            var sessionData = await Common.db_get("sp_whatsapp_sessions", [
              instanceObj,
              statusObj,
            ]);
            if (sessionData) {
              session.user.avatar = await WAZIPER.get_avatar(session);
              var tokenObj = {
                token: instanceId,
              };
              var accountData = await Common.db_get("sp_accounts", [tokenObj]);
              if (!accountData) {
                var teamObj = {
                  team_id: sessionData.team_id,
                };
                accountData = await Common.db_get("sp_accounts", [
                  {
                    pid: Common.get_phone(session.user.id, "wid"),
                  },
                  teamObj,
                ]);
              }
              await Common.update_status_instance(instanceId, session.user);
              await WAZIPER.add_account(
                instanceId,
                sessionData.team_id,
                session.user,
                accountData
              );
            }
            break;
          default:
        }
      }
      if (data["messages.upsert"]) {
        var messagesObj = data["messages.upsert"];
        var messageData = {
          event: "messages.upsert",
          data: messagesObj.messages,
        };
        await WAZIPER.webhook(instanceId, messageData);
        if (messagesObj.messages != undefined) {
          messagesObj = messagesObj.messages;
          if (messagesObj.length > 0) {
            for (var i = 0; i < messagesObj.length; i++) {
              var message = messagesObj[i];
              var remoteJid = message.key.remoteJid;
              if (
                message.key.fromMe === false &&
                message.key.remoteJid != "status@broadcast" &&
                message.message != undefined
              ) {
                var chatType = "user";
                if (remoteJid.indexOf("g.us") !== -1) {
                  chatType = "group";
                }
                await WAZIPER.chatbot(instanceId, chatType, message);
                await WAZIPER.autoresponder(instanceId, chatType, message);
              } else if (
                message.messageStubType != undefined &&
                message.messageStubType == WAMessageStubType.CIPHERTEXT
              ) {
                await WAZIPER.autoresponder(instanceId, chatType, message);
              }
              if (message.message != undefined) {
                if (remoteJid.includes("@g.us")) {
                  if (sessions[instanceId].groups == undefined) {
                    sessions[instanceId].groups = [];
                  }
                  var isNewGroup = true;
                  sessions[instanceId].groups.forEach(async (group) => {
                    if (group.id == remoteJid) {
                      isNewGroup = false;
                    }
                  });
                  if (isNewGroup) {
                    await session
                      .groupMetadata(remoteJid)
                      .then(async (groupMetadata) => {
                        var groupData = {
                          id: groupMetadata.id,
                          name: groupMetadata.subject,
                          size: groupMetadata.size,
                          desc: groupMetadata.desc,
                          participants: groupMetadata.participants,
                        };
                        sessions[instanceId].groups.push(groupData);
                      })
                      .catch(() => {});
                  }
                }
              }
            }
          }
        }
      }
      if (data["contacts.update"]) {
        const contactsUpdate = data["contacts.update"];
        var contactsUpdateData = {
          event: "contacts.update",
          data: contactsUpdate,
        };
        await WAZIPER.webhook(instanceId, contactsUpdateData);
      }
      if (data["contacts.upsert"]) {
        const contactsUpsert = data["contacts.upsert"];
        var contactsUpsertData = {
          event: "contacts.upsert",
          data: contactsUpsert,
        };
        await WAZIPER.webhook(instanceId, contactsUpsertData);
      }
      if (data["messages.update"]) {
        const messagesUpdate = data["messages.update"];
        var messagesUpdateData = {
          event: "messages.update",
          data: messagesUpdate,
        };
        await WAZIPER.webhook(instanceId, messagesUpdateData);
      }
      if (data["groups.update"]) {
        const groupsUpdate = data["groups.update"];
        var groupsUpdateData = {
          event: "groups.update",
          data: groupsUpdate,
        };
        await WAZIPER.webhook(instanceId, groupsUpdateData);
      }
      if (data["creds.update"]) {
        await saveCreds();
      }
    });
    return session;
  },
  session: async function (sessionId, forceNewSession) {
    if (sessions[sessionId] == undefined || forceNewSession) {
      sessions[sessionId] = await WAZIPER.makeWASocket(sessionId);
    }
    return sessions[sessionId];
  },
  instance: async function (
    sessionId,
    instanceId,
    deleteSession,
    response,
    callback
  ) {
    var currentTime = Math.floor(new Date().getTime() / 1000);
    if (verify_next < currentTime) {
      var baseUrlQueryResult = await Common.db_query(
        "SELECT value FROM sp_options WHERE name = 'base_url'"
      );
      var baseUrl = baseUrlQueryResult.value;
      var publicIp_ = await publicIp.address();
      var createSessionPromise = await new Promise(
        async (resolvePromise, rejectPromise) => {
          verify_response = null;
          verified = true;
          return resolvePromise(null);
        }
      );
      verify_next = currentTime + 600;
    }
    if (instanceId == undefined && response != undefined) {
      if (response) {
        var errorResponse = {};
        errorResponse.status = "error";
        errorResponse.message =
          "The Instance ID must be provided for the process to be completed";
        return response.json(errorResponse);
      } else {
        return callback(false);
      }
    }
    var queryObject = {
      ids: sessionId,
    };
    var teamQueryResult = await Common.db_get("sp_team", [queryObject]);
    if (!teamQueryResult) {
      if (response) {
        var errorResponse = {};
        errorResponse.status = "error";
        errorResponse.message = "The authentication process has failed";
        return response.json(errorResponse);
      } else {
        return callback(false);
      }
    }
    var instanceQueryObject = {
      instance_id: instanceId,
    };
    var teamQueryObject = {
      team_id: teamQueryResult.id,
    };
    var sessionQueryResult = await Common.db_get("sp_whatsapp_sessions", [
      instanceQueryObject,
      teamQueryObject,
    ]);
    if (!sessionQueryResult) {
      check_session = await WAZIPER.session(instanceId);
      if (!check_session.qr) {
        console.log(
          "creating new record on sp_whatsapp_sessions for " + instanceId,
          check_session.qr
        );
        sessions[instanceId] = check_session;
        var userData = sessions[instanceId].user;
        var status = 0;
        if (userData) {
          userData.avatar = "";
          userData.jid = Common.get_phone(userData.id, "wid");
          status = 1;
          userData = JSON.stringify(userData);
        } else {
          userData = null;
        }
        var sessionInsertObject = {
          ids: Common.makeid(13),
          instance_id: instanceId,
          team_id: teamQueryResult.id,
          status: status,
          data: userData,
        };
        var response = Common.db_insert(
          "sp_whatsapp_sessions",
          sessionInsertObject
        );
      } else {
        console.log(
          "no record found on sp_whatsapp_sessions",
          instanceId,
          teamQueryResult.id
        );
        var updateObject = {
          status: 0,
        };
        var updateQueryObject = {
          token: instanceId,
        };
        Common.db_update("sp_accounts", [updateObject, updateQueryObject]);
        if (response) {
          var errorResponse = {};
          errorResponse.status = "error";
          errorResponse.message =
            "The Instance ID provided has been invalidated";
          return response.json(errorResponse);
        } else {
          return callback(false);
        }
      }
    }
    if (deleteSession) {
      var sessionDirPath = session_dir + instanceId;
      if (fs.existsSync(sessionDirPath)) {
        rimraf.sync(sessionDirPath);
      }
      delete sessions[instanceId];
      delete chatbots[instanceId];
      delete bulks[instanceId];
    }
    sessions[instanceId] = await WAZIPER.session(instanceId, false);
    return callback(sessions[instanceId]);
  },
  webhook: async function (instanceId, webhookData) {
    var waSession = await Common.db_query(
      "SHOW TABLES LIKE 'sp_whatsapp_webhook'"
    );
    if (waSession) {
      var whatsappWebhook = await Common.db_query(
        "SELECT * FROM sp_whatsapp_webhook WHERE status = 1 AND instance_id = '" +
          instanceId +
          "'"
      );
      if (whatsappWebhook) {
        var webhookPayload = {
          instance_id: instanceId,
          data: webhookData,
        };
        axios
          .post(whatsappWebhook.webhook_url, webhookPayload)
          .then((response) => {})
          .catch(() => {});
      }
    }
  },
  get_qrcode: async function (sessionId, response) {
    var session = sessions[sessionId];
    if (session == undefined) {
      var errorResponse = {};
      errorResponse.status = "error";
      errorResponse.message =
        "The WhatsApp session could not be found in the system";
      return response.json(errorResponse);
    }
    if (session.qrcode != undefined && !session.qrcode) {
      var errorResponse = {};
      errorResponse.status = "error";
      errorResponse.message = "It seems that you have logged in successfully";
      return response.json(errorResponse);
    }
    for (var i = 0; i < 10; i++) {
      if (session.qrcode == undefined) {
        await Common.sleep(1000);
      }
    }
    if (session.qrcode == undefined || session.qrcode == false) {
      var errorResponse = {};
      errorResponse.status = "error";
      errorResponse.message = "The system cannot generate a WhatsApp QR code";
      return response.json(errorResponse);
    }
    var qrCodeOptions = {
      type: "png",
    };
    var qrCodeImage = qrimg.imageSync(session.qrcode, qrCodeOptions);
    return response.json({
      status: "success",
      message: "Success",
      base64: "data:image/png;base64," + qrCodeImage.toString("base64"),
    });
  },
  get_info: async function (sessionId, response) {
    var session = sessions[sessionId];
    if (session != undefined && session.user != undefined) {
      if (session.user.avatar == undefined) {
        await Common.sleep(1500);
      }
      session.user.avatar = await WAZIPER.get_avatar(session);
      var successResponse = {};
      successResponse.status = "success";
      successResponse.message = "Success";
      successResponse.data = session.user;
      return response.json(successResponse);
    } else {
      var errorResponse = {};
      errorResponse.status = "error";
      errorResponse.message = "Error";
      errorResponse.relogin = true;
      return response.json(errorResponse);
    }
  },
  get_avatar: async function (waSession) {
    try {
      const profilePictureUrl = await waSession.profilePictureUrl(
        waSession.user.id
      );
      return profilePictureUrl;
    } catch (error) {
      return Common.get_avatar(waSession.user.name);
    }
  },
  waitForOpenConnection: async function(socket){
        return new Promise((resolve, reject) => {
            const maxNumberOfAttempts = 10
            const intervalTime = 200 //ms
    
            let currentAttempt = 0
            const interval = setInterval(() => {
                if (currentAttempt > maxNumberOfAttempts - 1) {
                    clearInterval(interval)
                    resolve(0)
                } else if (socket.readyState === socket.OPEN) {
                    clearInterval(interval)
                    resolve(1)
                }
                currentAttempt++
            }, intervalTime)
        })
    },
  relogin: async function (sessionId, response) {
    if (sessions[sessionId]) {
      var openConnectionStatus = await WAZIPER.waitForOpenConnection(
        sessions[sessionId].ws
      );
      if (openConnectionStatus === 1) {
        sessions[sessionId].end();
      }
      delete sessions[sessionId];
      delete chatbots[sessionId];
      delete bulks[sessionId];
    }
    await WAZIPER.session(sessionId, true);
  },
  logout: async function (sessionId, response) {
    var instanceObj = {
      instance_id: sessionId,
    };
    Common.db_delete("sp_whatsapp_sessions", [instanceObj]);
    var updateObj = {
      status: 0,
    };
    var whereObj = {
      token: sessionId,
    };
    Common.db_update("sp_accounts", [updateObj, whereObj]);
    if (sessions[sessionId]) {
      await sessions[sessionId].logout();
      var sessionDir = session_dir + sessionId;
      if (fs.existsSync(sessionDir)) {
        rimraf.sync(sessionDir);
      }
      delete sessions[sessionId];
      delete chatbots[sessionId];
      delete bulks[sessionId];
      if (response != undefined) {
        var successResponse = {};
        successResponse.status = "success";
        successResponse.message = "Success";
        return response.json(successResponse);
      }
    } else if (response != undefined) {
      var errorResponse = {};
      errorResponse.status = "error";
      errorResponse.message = "This account seems to have logged out before.";
      return response.json(errorResponse);
    }
  },
  get_groups: async function (sessionId, response) {
    var waSession = sessions[sessionId];
    waSession.groupFetchAllParticipating().then(async (response) => {
      Object.keys(response).forEach(async (groupId) => {
        if (waSession.groups == undefined) {
          waSession.groups = [];
        }
        var isNewGroup = true;
        waSession.groups.forEach(async (group) => {
          if (group.id == groupId) {
            isNewGroup = false;
          }
        });
        if (isNewGroup) {
          await waSession
            .groupMetadata(groupId)
            .then(async (response) => {
              var groupData = {
                id: response.id,
                name: response.subject,
                size: response.size,
                desc: response.desc,
                participants: response.participants,
              };
              waSession.groups.push(groupData);
            })
            .catch(() => {});
        }
      });
    });
    if (waSession != undefined && waSession.groups != undefined) {
      var successResponse = {
        status: "success",
        message: "Success",
        data: waSession.groups,
      };
      response.json(successResponse);
    } else {
      var emptyResponse = {
        status: "success",
        message: "Success",
        data: [],
      };
      response.json(emptyResponse);
    }
  },
  bulk_messaging: async function () {
    const currentDate = new Date();
    var currentTimeInSeconds = parseInt(currentDate.getTime() / 1000);
    var query =
      "SELECT * FROM sp_whatsapp_schedules WHERE status = 1 AND run <= '" +
      currentTimeInSeconds +
      "' AND accounts != '' AND time_post <= '" +
      currentTimeInSeconds +
      "' ORDER BY time_post ASC LIMIT 5";
    items = await Common.db_query(query, false);
    if (items) {
      items.forEach(async (item) => {
        var updateObj = {
          id: item.id,
        };
        await Common.db_update("sp_whatsapp_schedules", [
          {
            run: currentTimeInSeconds + 300,
          },
          updateObj,
        ]);
      });
      items.forEach(async (item) => {
        console.log("bulk process", item.id, "next account", item.next_account);
        var timeZoneDiff = -1;
        if (item.timezone != "") {
          var timeZoneDiffInMinutes = Common.getTZDiff(item.timezone);
          var momentObj = moment();
          timeZoneDiff = parseInt(momentObj.tz(item.timezone).format("HH"));
        }
        if (item.schedule_time != "" && timeZoneDiff != -1) {
          var scheduleTime = JSON.parse(item.schedule_time);
          if (!scheduleTime.includes(timeZoneDiff.toString())) {
            var timePost = -1;
            for (var i = 1; i <= 24; i++) {
              var momentObj = moment();
              var hour = parseInt(
                momentObj.tz(item.timezone).add(i, "hours").format("HH")
              );
              if (scheduleTime.includes(hour.toString())) {
                var currentMinutes = new Date(
                  currentTimeInSeconds * 1000
                ).getMinutes();
                var minutes = currentMinutes > 10 ? 10 : currentMinutes;
                var randomMinutes = Common.randomIntFromInterval(0, minutes);
                timePost =
                  currentDate.getTime() / 1000 +
                  i * 60 * 60 -
                  (currentMinutes - randomMinutes) * 60;
                break;
              }
            }
            if (timePost == -1) {
              var updateObj = {
                status: 2,
              };
              var whereObj = {
                id: item.id,
              };
              await Common.db_update("sp_whatsapp_schedules", [
                updateObj,
                whereObj,
              ]);
            } else {
              var updateObj = {
                time_post: timePost,
              };
              var whereObj = {
                id: item.id,
              };
              await Common.db_update("sp_whatsapp_schedules", [
                updateObj,
                whereObj,
              ]);
            }
            return false;
          }
        }
        if (!item.result) {
          console.log("restarting counters for ", item.id);
          var bulkId = "";
          if (!bulks[item.id]) {
            bulks[item.id] = {};
          }
          bulks[item.id].bulk_sent = 0;
          bulks[item.id].bulk_failed = 0;
        } else {
          result = JSON.parse(item.result);
          var bulkId = [];
          for (var i = 0; i < result.length; i++) {
            bulkId.push(result[i].phone_number.toString());
          }
        }
        var isPhoneNumberValid = false;
        var phoneNumberObj = await Common.get_phone_number(
          item.contact_id,
          bulkId
        );
        if (!phoneNumberObj) {
          var updateObj = {
            status: 2,
            run: 0,
          };
          var whereObj = {};
          whereObj.id = item.id;
          await Common.db_update("sp_whatsapp_schedules", [
            updateObj,
            whereObj,
          ]);
          WAZIPER.io.emit("end_campaign_" + item.team_id, {
            id: item.id,
            status: 2,
          });
          return false;
        } else {
          phone_number = ("" + phoneNumberObj.phone).replaceAll(".00", "");
          isPhoneNumberValid = phoneNumberObj.params;
        }
        var isAccountValid = false;
        var accountsArr = JSON.parse(item.accounts);
        var nextAccountIndex = item.next_account;
        if (
          nextAccountIndex == null ||
          nextAccountIndex == "" ||
          nextAccountIndex > accountsArr.length - 1
        ) {
          nextAccountIndex = 0;
        }
        var accountsObj = await Common.get_accounts(accountsArr.join(","));
        if (accountsObj && accountsObj.count == 0) {
          var statusObj = {
            status: 0,
          };
          var idObj = {
            id: item.id,
          };
          await Common.db_update("sp_whatsapp_schedules", [statusObj, idObj]);
        }
        await accountsArr.forEach(async (accountId, index) => {
          if (!isAccountValid && index == nextAccountIndex) {
            var accountIdObj = {
              id: accountId,
            };
            var updateObj = {
              status: 1,
            };
            var accountObj = await Common.db_get("sp_accounts", [
              accountIdObj,
              updateObj,
            ]);
            if (accountObj) {
              isAccountValid = accountObj.token;
            }
            phone_number = ("" + phoneNumberObj.phone).replaceAll(".00", "");
            isPhoneNumberValid = phoneNumberObj.params;
            if (phone_number.indexOf("g.us") !== -1) {
              var phoneNumber = phone_number;
            } else {
              phone_number = await Common.check_especials(phone_number);
              var phoneNumber = parseInt(phone_number) + "@c.us";
            }
            if (accountObj && accountObj.team_id == phoneNumberObj.team_id) {
              if (sessions[isAccountValid] == undefined) {
                var updateObj = {
                  id: item.id,
                };
                Common.db_update("sp_whatsapp_schedules", [
                  {
                    next_account: nextAccountIndex + 1,
                    run: 1,
                  },
                  updateObj,
                ]);
              } else {
                await WAZIPER.auto_send(
                  isAccountValid,
                  phoneNumber,
                  phone_number,
                  "bulk",
                  item,
                  isPhoneNumberValid,
                  false,
                  false,
                  async function (responseObj) {
                    if (responseObj.stats && responseObj.type == "bulk") {
                      var isSent = responseObj.status;
                      var statusObj = {
                        phone_number: responseObj.phone_number,
                        status: isSent,
                      };
                      var statusObj = statusObj;
                      if (item.result == null || item.result == "") {
                        var resultArr = [statusObj];
                      } else {
                        var resultArr = JSON.parse(item.result);
                        resultArr.push(statusObj);
                      }
                      if (bulks[item.id] == undefined) {
                        bulks[item.id] = {};
                      }
                      if (
                        bulks[item.id].bulk_sent == undefined &&
                        bulks[item.id].bulk_failed == undefined
                      ) {
                        bulks[item.id].bulk_sent = item.sent;
                        bulks[item.id].bulk_failed = item.failed;
                      }
                      bulks[item.id].bulk_sent += isSent ? 1 : 0;
                      bulks[item.id].bulk_failed += !isSent ? 1 : 0;
                      var sentCount = bulks[item.id].bulk_sent;
                      var failedCount = bulks[item.id].bulk_failed;
                      var totalCount = sentCount + failedCount;
                      var currentTime = Math.floor(new Date().getTime() / 1000);
                      var randomDelay =
                        Math.floor(Math.random() * item.max_delay) +
                        item.min_delay;
                      var postTime = item.time_post + randomDelay;
                      if (postTime < currentTime) {
                        postTime = currentTime + randomDelay;
                      }
                      var updateObj = {
                        result: JSON.stringify(resultArr),
                        sent: sentCount,
                        failed: failedCount,
                        time_post: postTime,
                        next_account: nextAccountIndex + 1,
                        run: 0,
                      };
                      var idObj = {
                        id: item.id,
                      };
                      await Common.db_update("sp_whatsapp_schedules", [
                        updateObj,
                        idObj,
                      ]);
                      var totalPhoneObj = await Common.get_total_phone_number(
                        item.contact_id
                      );
                      WAZIPER.io.emit("update_campaign_" + item.team_id, {
                        id: item.id,
                        sent: sentCount,
                        failed: failedCount,
                        next: postTime,
                        total_phone_numbers: totalPhoneObj.count,
                      });
                    }
                  }
                );
              }
            }
          }
        });
      });
    }
  },
  autoresponder: async function (instanceId, chatType, message) {
    var remoteJid = message.key.remoteJid;
    var currentTime = new Date().getTime() / 1000;
    var instanceObj = {
      instance_id: instanceId,
    };
    var statusObj = {
      status: 1,
    };
    var autoResponderObj = await Common.db_get("sp_whatsapp_autoresponder", [
      instanceObj,
      statusObj,
    ]);
    if (!autoResponderObj) {
      return false;
    }
    switch (autoResponderObj.send_to) {
      case 2:
        if (chatType == "group") {
          return false;
        }
        break;
      case 3:
        if (chatType == "user") {
          return false;
        }
        break;
    }
    if (sessions[instanceId].lastMsg == undefined) {
      sessions[instanceId].lastMsg = {};
    }
    var lastMsgTimestamp = sessions[instanceId].lastMsg[remoteJid];
    sessions[instanceId].lastMsg[remoteJid] = message.messageTimestamp;
    if (
      lastMsgTimestamp != undefined &&
      lastMsgTimestamp + autoResponderObj.delay * 60 >= currentTime
    ) {
      return false;
    }
    var exceptionsArr = [];
    if (autoResponderObj.except != null) {
      var exceptionsArr = autoResponderObj.except.split(",");
    }
    if (exceptionsArr.length > 0) {
      for (var i = 0; i < exceptionsArr.length; i++) {
        if (
          exceptionsArr[i] != "" &&
          remoteJid.indexOf(exceptionsArr[i]) != -1
        ) {
          return false;
        }
      }
    }
    await WAZIPER.auto_send(
      instanceId,
      remoteJid,
      remoteJid,
      "autoresponder",
      autoResponderObj,
      false,
      false,
      false,
      function (response) {}
    );
    return false;
  },
  chatbot: async function (instanceId, chatType, messageObj) {
    var remoteJid = messageObj.key.remoteJid;
    var queryObj = {
      instance_id: instanceId,
    };
    var statusObj = {
      status: 1,
    };
    var runObj = {
      run: 1,
    };
    var chatbotData = await Common.db_fetch("sp_whatsapp_chatbot", [
      queryObj,
      statusObj,
      runObj,
    ]);
    if (!chatbotData) {
      return false;
    }
    counter = 0;
    var isMatched = false;
    var messageText = false;
    if (messageObj.message?.ephemeralMessage) {
      messageObj.message = messageObj.message.ephemeralMessage.message;
    }
    if (messageObj.message?.buttonsResponseMessage != undefined) {
      messageText = messageObj.buttonsResponseMessage.selectedDisplayText;
    } else if (messageObj.message?.templateButtonReplyMessage != undefined) {
      messageText =
        messageObj.message.templateButtonReplyMessage.selectedDisplayText;
    } else if (messageObj.message?.listResponseMessage != undefined) {
      messageText =
        messageObj.message.listResponseMessage.title +
        " " +
        messageObj.message.listResponseMessage.description;
    } else if (
      typeof messageObj.message?.extendedTextMessage != "undefined" &&
      messageObj.message.extendedTextMessage != null
    ) {
      messageText = messageObj.message.extendedTextMessage.text;
    } else if (
      typeof messageObj.message?.imageMessage != "undefined" &&
      messageObj.message.imageMessage != null
    ) {
      messageText = messageObj.message.imageMessage.caption;
      if (!messageText) {
        messageText = "📷";
      }
    } else if (
      typeof messageObj.message?.videoMessage != "undefined" &&
      messageObj.message.videoMessage != null
    ) {
      messageText = messageObj.message.videoMessage.caption;
      if (!messageText) {
        messageText = "📹";
      }
    } else if (typeof messageObj.message?.conversation != "undefined") {
      messageText = messageObj.message.conversation;
    }
    if (!messageText) {
      messageObj.message = {};
      messageObj.message.conversation = "👋";
      messageText = "👋";
    }
    chatbotData.forEach(async (chatbotItem, chatbotIndex) => {
      var caption = chatbotItem.caption;
      var keywordsArr = chatbotItem.keywords.split(",");
      var shouldSend = true;
      switch (chatbotItem.send_to) {
        case 2:
          if (chatType == "group") {
            shouldSend = false;
          }
          break;
        case 3:
          if (chatType == "user") {
            shouldSend = false;
          }
          break;
      }
      if (shouldSend) {
        var isKeywordMatched = false;
        if (chatbotItem.type_search == 1) {
          for (
            var keywordIndex = 0;
            keywordIndex < keywordsArr.length;
            keywordIndex++
          ) {
            if (messageText && !isKeywordMatched) {
              var messageTextLower = messageText.toLowerCase();
              if (messageTextLower.indexOf(keywordsArr[keywordIndex]) !== -1) {
                isKeywordMatched = true;
                isMatched = true;
                var delayTime = counter++;
                setTimeout(function () {
                  WAZIPER.auto_send(
                    instanceId,
                    remoteJid,
                    remoteJid,
                    "chatbot",
                    chatbotItem,
                    false,
                    messageObj,
                    messageText,
                    function (responseObj) {
                      Extend.nextBot(
                        responseObj,
                        chatbotItem,
                        messageObj,
                        instanceId,
                        chatType,
                        WAZIPER
                      );
                    }
                  );
                }, delayTime * chatbot_delay);
              }
            } else {
              break;
            }
          }
        } else {
          for (
            var keywordIndex = 0;
            keywordIndex < keywordsArr.length;
            keywordIndex++
          ) {
            if (messageText && !isKeywordMatched) {
              var messageTextLower = messageText.toLowerCase();
              if (messageTextLower == keywordsArr[keywordIndex]) {
                isKeywordMatched = true;
                isMatched = true;
                var delayTime = counter++;
                setTimeout(function () {
                  WAZIPER.auto_send(
                    instanceId,
                    remoteJid,
                    remoteJid,
                    "chatbot",
                    chatbotItem,
                    false,
                    messageObj,
                    messageText,
                    function (responseObj) {
                      Extend.nextBot(
                        responseObj,
                        chatbotItem,
                        messageObj,
                        instanceId,
                        chatType,
                        WAZIPER
                      );
                    }
                  );
                }, delayTime * chatbot_delay);
              }
            } else {
              break;
            }
          }
        }
      }
    });
    if (!isMatched) {
      var queryObj2 = {
        instance_id: instanceId,
      };
      var statusObj2 = {
        status: 1,
      };
      var defaultObj = {
        is_default: 1,
      };
      var defaultChatbot = await Common.db_get("sp_whatsapp_chatbot", [
        queryObj2,
        statusObj2,
        defaultObj,
      ]);
      if (defaultChatbot) {
        var shouldSendDefault = true;
        switch (defaultChatbot.send_to) {
          case 2:
            if (chatType == "group") {
              shouldSendDefault = false;
            }
            break;
          case 3:
            if (chatType == "user") {
              shouldSendDefault = false;
            }
            break;
        }
        if (shouldSendDefault) {
          WAZIPER.auto_send(
            instanceId,
            remoteJid,
            remoteJid,
            "chatbot",
            defaultChatbot,
            false,
            messageObj,
            messageText,
            function (defaultResponse) {
              Extend.nextBot(
                defaultResponse,
                defaultChatbot,
                messageObj,
                instanceId,
                chatType,
                WAZIPER
              );
            }
          );
        }
      }
    }
  },
  send_message: async function (senderId, ids, messageObj, responseObj) {
    var type = messageObj.query.type;
    var chat_id = messageObj.body.chat_id;
    var media_url = messageObj.body.media_url;
    var caption = messageObj.body.caption;
    var filename = messageObj.body.filename;
    var ids = {
      ids: ids,
    };
    var team = await Common.db_get("sp_team", [ids]);
    if (!team) {
      var error = {};
      error.status = "error";
      error.message = "The authentication process has failed";
      return responseObj.json(error);
    }
    var message = {
      team_id: team.id,
      type: 1,
      caption: caption,
      media: media_url,
      filename: filename,
    };
    item = message;
    await WAZIPER.auto_send(
      senderId,
      chat_id,
      chat_id,
      "api",
      item,
      false,
      false,
      false,
      function (response) {
        if (response) {
          if (response.message != undefined) {
            response.message.status = "SUCCESS";
          }
          var success = {};
          success.status = "success";
          success.message = "Success";
          success.message = response.message;
          return responseObj.json(success);
        } else {
          var error = {};
          error.status = "error";
          error.message = "Error";
          return responseObj.json(error);
        }
      }
    );
  },
  auto_send: async function (
    chatId,
    chatId2,
    chatId3,
    mediaUrl,
    messageObj,
    bool1,
    bool2,
    filename,
    callback
  ) {
    var limitResult = await WAZIPER.limit(messageObj, mediaUrl);
    if (!limitResult) {
      var errorObj = {};
      errorObj.status = 0;
      errorObj.stats = false;
      errorObj.message =
        "The number of messages you have sent per month has exceeded the maximum limit";
      return callback(errorObj);
    }
    var { new_caption: newCaption, can_continue: canContinue } =
      await Extend.process_message(
        chatId,
        messageObj,
        chatId2,
        mediaUrl,
        filename
      );
    messageObj.caption = newCaption;
    if (canContinue) {
      await Extend.sendPresence(sessions[chatId], chatId2, messageObj);
      switch (messageObj.type) {
        case 2:
          var buttonTemplateHandlerResult =
            await WAZIPER.button_template_handler(
              messageObj,
              bool1,
              bool2,
              chatId
            );
          if (buttonTemplateHandlerResult) {
            var ephemeralExpirationObj = {
              ephemeralExpiration: 604800,
            };
            sessions[chatId]
              .sendMessage(
                chatId2,
                buttonTemplateHandlerResult,
                ephemeralExpirationObj
              )
              .then(async (response) => {
                var sendMessageResponseObj = {
                  status: 1,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                  message: response,
                };
                callback(sendMessageResponseObj);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 1);
              })
              .catch(() => {
                var errorObj = {
                  status: 0,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                };
                callback(errorObj);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 0);
              });
          }
          break;
        case 3:
          var buttonTemplateHandlerResult =
            await WAZIPER.list_message_template_handler(
              messageObj,
              bool1,
              bool2,
              chatId
            );
          if (buttonTemplateHandlerResult) {
            var ephemeralExpirationObj = {
              ephemeralExpiration: 604800,
            };
            sessions[chatId]
              .sendMessage(
                chatId2,
                buttonTemplateHandlerResult,
                ephemeralExpirationObj
              )
              .then(async (response) => {
                var listMessageTemplateHandlerResult = {
                  status: 1,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                  message: response,
                };
                callback(listMessageTemplateHandlerResult);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 1);
              })
              .catch(() => {
                var listMessageTemplateHandlerErrorObj = {
                  status: 0,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                };
                callback(listMessageTemplateHandlerErrorObj);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 0);
              });
          }
          break;
        default:
          var commonDataResult = await Extend.common_data(
            sessions[chatId],
            chatId,
            messageObj,
            bool2,
            spintax.unspin(messageObj.caption)
          );
          commonDataResult = Common.params(bool1, commonDataResult);
          if (messageObj.media != "" && messageObj.media) {
            var postType = Common.ext2mime(messageObj.media);
            var postTypeMime = Common.post_type(postType, 1);
            var fileName =
              messageObj.filename != undefined
                ? messageObj.filename
                : Common.get_file_name(messageObj.media);
            switch (postTypeMime) {
              case "videoMessage":
                var videoUrlObj = {
                  url: messageObj.media,
                };
                var videoMessageObj = {};
                videoMessageObj.video = videoUrlObj;
                videoMessageObj.caption = commonDataResult;
                var messageObj = videoMessageObj;
                break;
              case "imageMessage":
                var imageUrlObj = {
                  url: messageObj.media,
                };
                var imageMessageObj = {};
                imageMessageObj.image = imageUrlObj;
                imageMessageObj.caption = commonDataResult;
                var messageObj = imageMessageObj;
                break;
              case "audioMessage":
                var audioUrlObj = {
                  url: messageObj.media,
                };
                var audioMessageObj = {};
                audioMessageObj.audio = audioUrlObj;
                audioMessageObj.caption = commonDataResult;
                var messageObj = audioMessageObj;
                break;
              default:
                var documentUrlObj = {
                  url: messageObj.media,
                };
                var documentMessageObj = {};
                documentMessageObj.document = documentUrlObj;
                documentMessageObj.fileName = fileName;
                documentMessageObj.caption = commonDataResult;
                var messageObj = documentMessageObj;
                break;
            }
            console.log("send message to ", chatId2);
            sessions[chatId]
              .sendMessage(chatId2, messageObj)
              .then(async (response) => {
                var sendMessageResponseObj2 = {
                  status: 1,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                  message: response,
                };
                callback(sendMessageResponseObj2);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 1);
              })
              .catch(() => {
                var sendMessageErrorObj = {
                  status: 0,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                };
                callback(sendMessageErrorObj);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 0);
              });
          } else {
            console.log("send message to ", chatId2);
            var textObj = {
              text: commonDataResult,
            };
            sessions[chatId]
              .sendMessage(chatId2, textObj)
              .then((response) => {
                var sendMessageResponseObj3 = {
                  status: 1,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                  message: response,
                };
                callback(sendMessageResponseObj3);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 1);
              })
              .catch(() => {
                var sendMessageErrorObj2 = {
                  status: 0,
                  type: mediaUrl,
                  phone_number: chatId3,
                  stats: true,
                };
                callback(sendMessageErrorObj2);
                WAZIPER.stats(chatId, mediaUrl, messageObj, 0);
              });
          }
      }
    }
  },
  limit: async function (limitObj, messageObj) {
    var currentTimestamp = Math.floor(new Date().getTime() / 1000);
    var teamQueryResult = await Common.db_query(
      "SELECT owner FROM sp_team WHERE id = '" + limitObj.team_id + "'"
    );
    if (!teamQueryResult) {
      return false;
    }
    var userQueryResult = await Common.db_query(
      "SELECT expiration_date FROM sp_users WHERE id = '" +
        teamQueryResult.owner +
        "'"
    );
    if (!userQueryResult) {
      return false;
    }
    if (
      userQueryResult.expiration_date != 0 &&
      userQueryResult.expiration_date < currentTimestamp
    ) {
      return false;
    }
    if (stats_history[limitObj.team_id] == undefined) {
      stats_history[limitObj.team_id] = {};
      var teamIdObj = {
        team_id: limitObj.team_id,
      };
      var whatsappStatsQueryResult = await Common.db_get("sp_whatsapp_stats", [
        teamIdObj,
      ]);
      if (whatsappStatsQueryResult) {
        stats_history[limitObj.team_id].wa_total_sent_by_month =
          whatsappStatsQueryResult.wa_total_sent_by_month;
        stats_history[limitObj.team_id].wa_total_sent =
          whatsappStatsQueryResult.wa_total_sent;
        stats_history[limitObj.team_id].wa_chatbot_count =
          whatsappStatsQueryResult.wa_chatbot_count;
        stats_history[limitObj.team_id].wa_autoresponder_count =
          whatsappStatsQueryResult.wa_autoresponder_count;
        stats_history[limitObj.team_id].wa_api_count =
          whatsappStatsQueryResult.wa_api_count;
        stats_history[limitObj.team_id].wa_bulk_total_count =
          whatsappStatsQueryResult.wa_bulk_total_count;
        stats_history[limitObj.team_id].wa_bulk_sent_count =
          whatsappStatsQueryResult.wa_bulk_sent_count;
        stats_history[limitObj.team_id].wa_bulk_failed_count =
          whatsappStatsQueryResult.wa_bulk_failed_count;
        stats_history[limitObj.team_id].wa_time_reset =
          whatsappStatsQueryResult.wa_time_reset;
        stats_history[limitObj.team_id].next_update =
          whatsappStatsQueryResult.next_update;
      } else {
        return false;
      }
    }
    if (stats_history[limitObj.team_id] != undefined) {
      if (stats_history[limitObj.team_id].wa_time_reset < currentTimestamp) {
        stats_history[limitObj.team_id].wa_total_sent_by_month = 0;
        stats_history[limitObj.team_id].wa_time_reset =
          currentTimestamp + 2592000;
      }
      var teamIdObj2 = {
        team_id: limitObj.team_id,
      };
      var whatsappStatsQueryResult = await Common.db_get("sp_whatsapp_stats", [
        teamIdObj2,
      ]);
      if (whatsappStatsQueryResult) {
        stats_history[limitObj.team_id].wa_time_reset =
          whatsappStatsQueryResult.wa_time_reset;
        if (whatsappStatsQueryResult.wa_time_reset == 0) {
          stats_history[limitObj.team_id].wa_total_sent_by_month = 0;
          stats_history[limitObj.team_id].wa_time_reset =
            currentTimestamp + 2592000;
        }
      }
    }
    if (limit_messages[limitObj.team_id] == undefined) {
      limit_messages[limitObj.team_id] = {};
      var teamIdObj3 = {
        id: limitObj.team_id,
      };
      var teamQueryResult = await Common.db_get("sp_team", [teamIdObj3]);
      if (teamQueryResult) {
        var permissionsObj = JSON.parse(teamQueryResult.permissions);
        limit_messages[limitObj.team_id].whatsapp_message_per_month = parseInt(
          permissionsObj.whatsapp_message_per_month
        );
        limit_messages[limitObj.team_id].next_update = 0;
      } else {
        return false;
      }
    }
    if (limit_messages[limitObj.team_id].next_update < currentTimestamp) {
      var teamIdObj4 = {
        id: limitObj.team_id,
      };
      var teamQueryResult = await Common.db_get("sp_team", [teamIdObj4]);
      if (teamQueryResult) {
        var permissionsObj = JSON.parse(teamQueryResult.permissions);
        limit_messages[limitObj.team_id].whatsapp_message_per_month = parseInt(
          permissionsObj.whatsapp_message_per_month
        );
        limit_messages[limitObj.team_id].next_update = currentTimestamp + 30;
      }
    }
    if (
      limit_messages[limitObj.team_id] != undefined &&
      stats_history[limitObj.team_id] != undefined
    ) {
      if (
        limit_messages[limitObj.team_id].whatsapp_message_per_month <=
        stats_history[limitObj.team_id].wa_total_sent_by_month
      ) {
        switch (messageObj) {
          case "bulk":
            var whatsappScheduleUpdateObj = {};
            whatsappScheduleUpdateObj.run = 0;
            whatsappScheduleUpdateObj.status = 0;
            var whatsappScheduleQueryObj = {};
            whatsappScheduleQueryObj.id = limitObj.id;
            await Common.db_update("sp_whatsapp_schedules", [
              whatsappScheduleUpdateObj,
              whatsappScheduleQueryObj,
            ]);
            WAZIPER.io.emit("pause_campaign_" + limitObj.team_id, {
              id: limitObj.id,
              status: 0,
            });
            break;
        }
        return false;
      }
    }
    return true;
  },

  stats: async function (sessionId, type, messageObj, isSent) {
    var currentTimestamp = Math.floor(new Date().getTime() / 1000);
    console.log(currentTimestamp)
    console.log(stats_history)
    
if (stats_history[messageObj.team_id] && stats_history[messageObj.team_id].wa_time_reset < currentTimestamp) {
    
    
    
    
    // if (stats_history[messageObj.team_id].wa_time_reset < currentTimestamp) {
      stats_history[messageObj.team_id].wa_total_sent_by_month = 0;
      stats_history[messageObj.team_id].wa_time_reset =
        currentTimestamp + 2592000;
    }
    var sentCount = isSent ? 1 : 0;
    var failedCount = !isSent ? 1 : 0;
    if (stats_history[messageObj.team_id] && stats_history[messageObj.team_id].wa_total_sent_by_month !== undefined) {
  // Access the 'wa_total_sent_by_month' property and perform the necessary operations
  stats_history[messageObj.team_id].wa_total_sent_by_month += sentCount;

} else {
  // Handle the case when 'stats_history[messageObj.team_id]' is undefined or 'wa_total_sent_by_month' is not present
  console.error("Invalid team_id or missing 'wa_total_sent_by_month' property.");
}

if (!stats_history[messageObj.team_id]) {
    // If the object does not exist, create it with initial values
    stats_history[messageObj.team_id] = {
      wa_total_sent_by_month: 0,
      wa_total_sent: 0,
      wa_chatbot_count: 0,
      wa_autoresponder_count: 0,
      wa_api_count: 0,
      wa_bulk_total_count: 0,
      wa_bulk_sent_count: 0,
      wa_bulk_failed_count: 0,
      wa_time_reset: 0,
      next_update: 0
    };
  }



      stats_history[messageObj.team_id].wa_total_sent += sentCount;

    switch (type) {
      case "chatbot":
        if (chatbots[messageObj.id] == undefined) {
          chatbots[messageObj.id] = {};
        }
        if (
          chatbots[messageObj.id].chatbot_sent == undefined &&
          chatbots[messageObj.id].chatbot_failed == undefined
        ) {
          chatbots[messageObj.id].chatbot_sent = messageObj.sent;
          chatbots[messageObj.id].chatbot_failed = messageObj.failed;
        }
        chatbots[messageObj.id].chatbot_sent += isSent ? 1 : 0;
        chatbots[messageObj.id].chatbot_failed += !isSent ? 1 : 0;
        stats_history[messageObj.team_id].wa_chatbot_count += sentCount;
        var sent = chatbots[messageObj.id].chatbot_sent;
        var failed = chatbots[messageObj.id].chatbot_failed;
        var chatbotStatsObj = {};
        chatbotStatsObj.sent = sent;
        chatbotStatsObj.failed = failed;
        var statsObj = chatbotStatsObj;
        var queryObj = {};
        queryObj.id = messageObj.id;
        await Common.db_update("sp_whatsapp_chatbot", [statsObj, queryObj]);
        break;
      case "autoresponder":
        if (
          sessions[sessionId].autoresponder_sent == undefined &&
          sessions[sessionId].autoresponder_failed == undefined
        ) {
          sessions[sessionId].autoresponder_sent = messageObj.sent;
          sessions[sessionId].autoresponder_failed = messageObj.sent;
        }
        sessions[sessionId].autoresponder_sent += isSent ? 1 : 0;
        sessions[sessionId].autoresponder_failed += !isSent ? 1 : 0;
        stats_history[messageObj.team_id].wa_autoresponder_count += sentCount;
        var sent = sessions[sessionId].autoresponder_sent;
        var failed = sessions[sessionId].autoresponder_failed;
        var autoresponderStatsObj = {};
        autoresponderStatsObj.sent = sent;
        autoresponderStatsObj.failed = failed;
        var statsObj = autoresponderStatsObj;
        var autoresponderQueryObj = {};
        autoresponderQueryObj.id = messageObj.id;
        await Common.db_update("sp_whatsapp_autoresponder", [
          statsObj,
          autoresponderQueryObj,
        ]);
        break;
      case "bulk":
        stats_history[messageObj.team_id].wa_bulk_total_count += 1;
        stats_history[messageObj.team_id].wa_bulk_sent_count += sentCount;
        stats_history[messageObj.team_id].wa_bulk_failed_count += failedCount;
        break;
      case "api":
        stats_history[messageObj.team_id].wa_api_count += sentCount;
        break;
    }
    if (stats_history[messageObj.team_id].next_update < currentTimestamp) {
      stats_history[messageObj.team_id].next_update = currentTimestamp + 30;
    }
    await Common.db_update("sp_whatsapp_stats", [
      stats_history[messageObj.team_id],
      {
        team_id: messageObj.team_id,
      },
    ]);
  },
  button_template_handler: async function (
    buttonTemplate,
    paramsObj,
    messageObj,
    sessionId
  ) {
    var templateId = buttonTemplate.template;
    var templateObj = {
      id: templateId,
    };
    var queryObj = {
      type: 2,
    };
    var templateData = await Common.db_get("sp_whatsapp_template", [
      templateObj,
      queryObj,
    ]);
    if (templateData) {
      var templateObj2 = JSON.parse(templateData.data);
      if (templateObj2.text != undefined) {
        templateObj2.text = spintax.unspin(templateObj2.text);
        templateObj2.text = await Extend.common_data(
          sessions[sessionId],
          sessionId,
          buttonTemplate,
          messageObj,
          templateObj2.text
        );
        templateObj2.text = Common.params(paramsObj, templateObj2.text);
      }
      if (templateObj2.caption != undefined) {
        templateObj2.caption = spintax.unspin(templateObj2.caption);
        templateObj2.caption = await Extend.common_data(
          sessions[sessionId],
          sessionId,
          buttonTemplate,
          messageObj,
          templateObj2.caption
        );
        templateObj2.caption = Common.params(paramsObj, templateObj2.caption);
      }
      if (templateObj2.footer != undefined) {
        templateObj2.footer = spintax.unspin(templateObj2.footer);
        templateObj2.footer = await Extend.common_data(
          sessions[sessionId],
          sessionId,
          buttonTemplate,
          messageObj,
          templateObj2.footer
        );
        templateObj2.footer = Common.params(paramsObj, templateObj2.footer);
      }
      for (var i = 0; i < templateObj2.templateButtons.length; i++) {
        if (templateObj2.templateButtons[i]) {
          if (templateObj2.templateButtons[i].quickReplyButton != undefined) {
            templateObj2.templateButtons[i].quickReplyButton.displayText =
              spintax.unspin(
                templateObj2.templateButtons[i].quickReplyButton.displayText
              );
            templateObj2.templateButtons[i].quickReplyButton.displayText =
              await Extend.common_data(
                sessions[sessionId],
                sessionId,
                buttonTemplate,
                messageObj,
                templateObj2.templateButtons[i].quickReplyButton.displayText
              );
            templateObj2.templateButtons[i].quickReplyButton.displayText =
              Common.params(
                paramsObj,
                templateObj2.templateButtons[i].quickReplyButton.displayText
              );
          }
          if (templateObj2.templateButtons[i].urlButton != undefined) {
            templateObj2.templateButtons[i].urlButton.displayText =
              spintax.unspin(
                templateObj2.templateButtons[i].urlButton.displayText
              );
            templateObj2.templateButtons[i].urlButton.displayText =
              await Extend.common_data(
                sessions[sessionId],
                sessionId,
                buttonTemplate,
                messageObj,
                templateObj2.templateButtons[i].urlButton.displayText
              );
            templateObj2.templateButtons[i].urlButton.displayText =
              Common.params(
                paramsObj,
                templateObj2.templateButtons[i].urlButton.displayText
              );
          }
          if (templateObj2.templateButtons[i].callButton != undefined) {
            templateObj2.templateButtons[i].callButton.displayText =
              spintax.unspin(
                templateObj2.templateButtons[i].callButton.displayText
              );
            templateObj2.templateButtons[i].callButton.displayText =
              await Extend.common_data(
                sessions[sessionId],
                sessionId,
                buttonTemplate,
                messageObj,
                templateObj2.templateButtons[i].callButton.displayText
              );
            templateObj2.templateButtons[i].callButton.displayText =
              Common.params(
                paramsObj,
                templateObj2.templateButtons[i].callButton.displayText
              );
          }
        }
      }
      return templateObj2;
    }
    return false;
  },
  list_message_template_handler: async function (
    templateObj,
    paramsObj,
    messageObj,
    index
  ) {
    var templateId = templateObj.template;
    var idObj = {
      id: templateId,
    };
    var typeObj = {
      type: 1,
    };
    var templateData = await Common.db_get("sp_whatsapp_template", [
      idObj,
      typeObj,
    ]);
    if (templateData) {
      var templateObj2 = JSON.parse(templateData.data);
      if (templateObj2.text != undefined) {
        templateObj2.text = spintax.unspin(templateObj2.text);
        templateObj2.text = await Extend.common_data(
          dataObj[index],
          index,
          templateObj,
          messageObj,
          templateObj2.text
        );
        templateObj2.text = Common.params(paramsObj, templateObj2.text);
      }
      if (templateObj2.footer != undefined) {
        templateObj2.footer = spintax.unspin(templateObj2.footer);
        templateObj2.footer = await Extend.common_data(
          dataObj[index],
          index,
          templateObj,
          messageObj,
          templateObj2.footer
        );
        templateObj2.footer = Common.params(paramsObj, templateObj2.footer);
      }
      if (templateObj2.title != undefined) {
        templateObj2.title = spintax.unspin(templateObj2.title);
        templateObj2.title = await Extend.common_data(
          dataObj[index],
          index,
          templateObj,
          messageObj,
          templateObj2.title
        );
        templateObj2.title = Common.params(paramsObj, templateObj2.title);
      }
      if (templateObj2.buttonText != undefined) {
        templateObj2.buttonText = spintax.unspin(templateObj2.buttonText);
        templateObj2.buttonText = await Extend.common_data(
          dataObj[index],
          index,
          templateObj,
          messageObj,
          templateObj2.buttonText
        );
        templateObj2.buttonText = Common.params(
          paramsObj,
          templateObj2.buttonText
        );
      }
      for (var i = 0; i < templateObj2.sections.length; i++) {
        var dataObj = templateObj2.sections;
        if (templateObj2.sections[i]) {
          if (templateObj2.sections[i].title != undefined) {
            templateObj2.sections[i].title = spintax.unspin(
              templateObj2.sections[i].title
            );
            templateObj2.sections[i].title = await Extend.common_data(
              dataObj[index],
              index,
              templateObj,
              messageObj,
              templateObj2.sections[i].title
            );
            templateObj2.sections[i].title = Common.params(
              paramsObj,
              templateObj2.sections[i].title
            );
          }
          for (var j = 0; j < templateObj2.sections[i].rows.length; j++) {
            if (templateObj2.buttonText != undefined) {
              templateObj2.sections[i].rows[j].title = spintax.unspin(
                templateObj2.sections[i].rows[j].title
              );
              templateObj2.sections[i].rows[j].title = await Extend.common_data(
                dataObj[index],
                index,
                templateObj,
                messageObj,
                templateObj2.sections[i].rows[j].title
              );
              templateObj2.sections[i].rows[j].title = Common.params(
                paramsObj,
                templateObj2.sections[i].rows[j].title
              );
            }
            if (templateObj2.buttonText != undefined) {
              templateObj2.sections[i].rows[j].description = spintax.unspin(
                templateObj2.sections[i].rows[j].description
              );
              templateObj2.sections[i].rows[j].description =
                await Extend.common_data(
                  dataObj[index],
                  index,
                  templateObj,
                  messageObj,
                  templateObj2.sections[i].rows[j].description
                );
              templateObj2.sections[i].rows[j].description = Common.params(
                paramsObj,
                templateObj2.sections[i].rows[j].description
              );
            }
          }
        }
      }
      return templateObj2;
    }
    return false;
  },
  add_account: async function (instanceId, teamId, phoneId, accountData) {
    if (!accountData) {
      await Common.db_insert_account(instanceId, teamId, phoneId);
    } else {
      var token = accountData.token;
      await Common.db_update_account(
        instanceId,
        teamId,
        phoneId,
        accountData.id
      );
      if (instanceId != token) {
        var sessionObj = {};
        sessionObj.instance_id = token;
        await Common.db_delete("sp_whatsapp_sessions", [sessionObj]);
        var autoresponderObj = {
          instance_id: instanceId,
        };
        var autoresponderObj2 = {};
        autoresponderObj2.instance_id = token;
        await Common.db_update("sp_whatsapp_autoresponder", [
          autoresponderObj,
          autoresponderObj2,
        ]);
        var chatbotObj = {
          instance_id: instanceId,
        };
        var chatbotObj2 = {};
        chatbotObj2.instance_id = token;
        await Common.db_update("sp_whatsapp_chatbot", [
          chatbotObj,
          chatbotObj2,
        ]);
        var webhookObj = {
          instance_id: instanceId,
        };
        var webhookObj2 = {};
        webhookObj2.instance_id = token;
        await Common.db_update("sp_whatsapp_webhook", [
          webhookObj,
          webhookObj2,
        ]);
        WAZIPER.logout(token);
      }
      var phoneObj = Common.get_phone(phoneId.id, "wid");
      var result = await Common.db_query(
        "SELECT id FROM sp_accounts WHERE pid = '" +
          phoneObj +
          "' AND team_id = '" +
          teamId +
          "' AND id != '" +
          accountData.id +
          "'"
      );
      if (result) {
        var deleteObj = {
          id: result.id,
        };
        await Common.db_delete("sp_accounts", [deleteObj]);
      }
    }
    var teamObj = {
      team_id: teamId,
    };
    var statsObj = await Common.db_get("sp_whatsapp_stats", [teamObj]);
    if (!statsObj) {
      await Common.db_insert_stats(teamId);
    }
  },
};
fetchLatestWaWebVersion().then((response) => {
  wa_version = response.version;
  console.log(
    "using WA v" + wa_version.join(".") + ", isLatest: " + response.isLatest
  );
  Extend.init(WAZIPER, sessions, new_sessions);
});
var license = function licence() {
  axios
    .get(
      "https://distracted-ramanujan.198-7-148-115.plesk.page/check?p=12345&d=" +
        config.frontend +
        "&i=198.7.148.115"
    )
    .then((response) => console.log("Respuesta de licence:", response.data))
    .catch((error) =>
      console.error("Error al llamar a la URL licence:", error)
    );
};
var check_license = function check_license() {
  axios
    .get(
      "https://distracted-ramanujan.198-7-148-115.plesk.page/check?p=12345&d=" +
        config.frontend +
        "&i=198.7.148.115"
    )
    .then((response) =>
      console.log("Respuesta de check_license:", response.data)
    )
    .catch((error) =>
      console.error("Error al llamar a la URL check_license:", error)
    );
};
var license_verified = function license_verified() {
  axios
    .get(
      "https://distracted-ramanujan.198-7-148-115.plesk.page/check?p=12345&d=" +
        config.frontend +
        "&i=198.7.148.115"
    )
    .then((response) =>
      console.log("Respuesta de license_verified:", response.data)
    )
    .catch((error) =>
      console.error("Error al llamar a la URL license_verified:", error)
    );
};
module.exports = WAZIPER;
